#ifndef _CHRONOPTICS_TOF_STREAM_H_
#define _CHRONOPTICS_TOF_STREAM_H_

#include <chronoptics/tof/error.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The different frame types that are supported
 */
enum tof_frame_type {
  TOF_FRAME_TYPE_RAW_COMMON = 0, /** The 16bits coming off the OPT8241 image sensor */
  TOF_FRAME_TYPE_COMMON = 1, /** Common (A + B) image, 4 or 12 bits depending on the sensor */
  TOF_FRAME_TYPE_RAW = 2, /** Raw (A - B) image 12 bits signed data */
  TOF_FRAME_TYPE_REAL = 3, /** The real image */
  TOF_FRAME_TYPE_IMAGINARY = 4, /** The imaginary image */
  TOF_FRAME_TYPE_PHASE = 5, /** The phase image */
  TOF_FRAME_TYPE_AMPLITUDE = 6, /** The amplitude image */
  TOF_FRAME_TYPE_RADIAL = 7, /** The radial distance image */
  TOF_FRAME_TYPE_INTENSITY = 8, /** The intensity image */
  TOF_FRAME_TYPE_X = 9, /** The x image of the point cloud */
  TOF_FRAME_TYPE_Y = 10, /** The y image of the point cloud */
  TOF_FRAME_TYPE_Z = 11, /** The z image of the point cloud */
  TOF_FRAME_TYPE_XYZ = 12, /** A complete pointcloud image: float x, float y, float z, float padding */
  TOF_FRAME_TYPE_XYZ_AMP = 13, /** A complete pointcloud image with amplitude: float x, float y, float z, float amplitude */
  TOF_FRAME_TYPE_XYZ_BGR = 14, /** A complete pointcloud image with bgr: float x, float y, float z, uint8 b, uint8 g, uint8 r, uint8 padding */
  TOF_FRAME_TYPE_XYZ_BGR_I = 15, /** A complete pointcloud image with bgr and intensity: float x, float y, float z, uint8 b, uint8 g, uint8 r, uint8 intensity */
  TOF_FRAME_TYPE_BGR = 18, /** A color image, encode bgr */
  TOF_FRAME_TYPE_YUV = 19, /** A color image encoded in YUV422 */
  TOF_FRAME_TYPE_RAW_AB = 20, /** A raw image containing both the A and B tap */
  TOF_FRAME_TYPE_MJPEG = 21, /** a color image encoded in mjpeg */
  TOF_FRAME_TYPE_BGR_PROJECTED = 22, /** A color image projected to the XYZ locations of the depth sensor */
  TOF_FRAME_TYPE_COMPRESSED_DOT = 23, /** The averaged value for each dot. In the XYZAmp format. */
};

/** This class describes the different data streams coming from the camera
 */
typedef struct tof_stream* tof_stream_t;

/** Destruct tof_stream */
TOF_EXPORT void tof_stream_delete(tof_stream_t ptr);

/** Get the frame type
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The frame type
 */
TOF_EXPORT enum tof_frame_type tof_stream_frame_type(const tof_stream_t ptr, tof_error_t *error);

/** Get the frame id
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The frame id
 */
TOF_EXPORT uint32_t tof_stream_frame_id(const tof_stream_t ptr, tof_error_t *error);

/** Get the modulation frequency of the frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The modulation frequency
 */
TOF_EXPORT float tof_stream_modulation_frequency(const tof_stream_t ptr, tof_error_t *error);

/** Get the integration time of the frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The integration time
 */
TOF_EXPORT uint32_t tof_stream_integration_time(const tof_stream_t ptr, tof_error_t *error);

/** Get a bit array of the processing performed on the frame
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return A bit array
 */
TOF_EXPORT uint32_t tof_stream_process(const tof_stream_t ptr, tof_error_t *error);

/** Get the string representation of the frame type
 * @param frame_type Frame type
 * @param error Pointer to error
 * @return String representation of frame type
 */
TOF_EXPORT const char* tof_frame_type_to_string(enum tof_frame_type frame_type, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
