import ctypes
from typing import List, Optional
from .__library__ import lib, get_error


class FlyingShape:
    """
    The shape of which surrounding pixels are checked in the flying pixel filter
    """

    PLUS = 0
    "Four neighboring pixels in a plus shape"

    BOX = 1
    "All eight neighboring pixels"



class MpSortMode:
    """
    Mixed pixel sort mode. The mode you want to use depends on your application.
    """

    DARKEST = 0
    "Choose darkest path"

    SHORTEST = 1
    "Return the surface of a translucent object, resolve corners or checkerboard lens flare"

    BRIGHTEST = 2
    "Mixed pixels between foreground and background, get the best SNR"

    LONGEST = 3
    "Correct lens flare from close bright objects"

    AGGRESSOR = 4
    "Correct lens flare"

    ADAPTIVE_SHORT_V0 = 5
    "Adaptive shortest"

    ADAPTIVE_SHORT_V1 = 6
    "Adaptive shortest version two"

    AGGRESSOR_V2 = 7
    "Correct lens flare version two"



class BinningMode:
    """
    What kind of binning to do
    """

    NORMAL = 0
    "Normal binning, a BxB grid averaged to a single pixel"

    SMART = 1
    "Smart binning, take into account the standard deviation around a chosen pixel. Similar to local means but reduces the image size"



class XyzDimension:
    """
    In what dimension the xyz is exported
    """

    MILLIMETER = 0
    "Export in millimeters"

    METER = 1
    "Export in meters"



class ProcessingConfig():
    """
    Processing that can be done
    """
    def __del__(self):
        if self._ptr:
            lib.tof_processing_config_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, file_location: Optional[str] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new processing_config class
        ----------

        Create default processing config
        ----------

        Load processing config from disk

        Parameters
        ----------
        file_location : str
                Location of processing config
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif file_location != None:
            self._ptr = lib.tof_processing_config_new_from_disk(file_location.encode("utf-8"), get_error())
        else:
            self._ptr = lib.tof_processing_config_new_default(get_error())


    def write(self, file_location: str) -> None:
        """
        Write processing config to disk

        Parameters
        ----------
        file_location : str
            Location where to save the processing config
        """
        return lib.tof_processing_config_write(self._ptr, file_location.encode("utf-8"), get_error())


    def getGpu(self) -> bool:
        """
        Get whether processing on the gpu is enabled

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_gpu(self._ptr, get_error())


    def setGpu(self, gpu: bool) -> None:
        """
        Set whether processing on the gpu is enabled

        Parameters
        ----------
        gpu : bool
            Enabled
        """
        return lib.tof_processing_config_set_gpu(self._ptr, gpu, get_error())


    def getCalibrationEnabled(self) -> bool:
        """
        Get whether the calibration is applied

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_calibration_enabled(self._ptr, get_error())


    def setCalibrationEnabled(self, enabled: bool) -> None:
        """
        Set whether the calibration is applied

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_calibration_enabled(self._ptr, enabled, get_error())


    def getPhaseUnwrappingEnabled(self) -> bool:
        """
        Get whether phase unwrapping is enabled

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_phase_unwrapping_enabled(self._ptr, get_error())


    def setPhaseUnwrappingEnabled(self, enabled: bool) -> None:
        """
        Set whether phase unwrapping is enabled

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_phase_unwrapping_enabled(self._ptr, enabled, get_error())


    def getPhaseUnwrappingMaxOffset(self) -> float:
        """
        Get The maximum distance between ideal location and actual

        Returns
        ----------
        float
            Max offset, range [0, 1]
        """
        return lib.tof_processing_config_get_phase_unwrapping_max_offset(self._ptr, get_error())


    def setPhaseUnwrappingMaxOffset(self, max_offset: float) -> None:
        """
        Set The maximum distance between ideal location and actual

        Parameters
        ----------
        max_offset : float
            Max offset, range [0, 1]
        """
        return lib.tof_processing_config_set_phase_unwrapping_max_offset(self._ptr, max_offset, get_error())


    def getMpiDetectEnabled(self) -> bool:
        """
        Get wether mpi detect is enabled

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_mpi_detect_enabled(self._ptr, get_error())


    def setMpiDetectEnabled(self, enabled: bool) -> None:
        """
        Set wether mpi detect is enabled

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_mpi_detect_enabled(self._ptr, enabled, get_error())


    def getMpiDetectAratio(self) -> float:
        """
        Get The amplitude ratio value

        Returns
        ----------
        float
            The maximum ratio
        """
        return lib.tof_processing_config_get_mpi_detect_aratio(self._ptr, get_error())


    def setMpiDetectAratio(self, aratio: float) -> None:
        """
        Set The amplitude ratio value

        Parameters
        ----------
        aratio : float
            The maximum ratio
        """
        return lib.tof_processing_config_set_mpi_detect_aratio(self._ptr, aratio, get_error())


    def getMpiDetectPhiDiff(self) -> float:
        """
        Get the phase difference ratio

        Returns
        ----------
        float
            The maximum phase difference
        """
        return lib.tof_processing_config_get_mpi_detect_phi_diff(self._ptr, get_error())


    def setMpiDetectPhiDiff(self, phi_diff: float) -> None:
        """
        Set the phase difference ratio

        Parameters
        ----------
        phi_diff : float
            The maximum phase difference
        """
        return lib.tof_processing_config_set_mpi_detect_phi_diff(self._ptr, phi_diff, get_error())


    def getMixedPixelEnabled(self) -> bool:
        """
        Get wether mixed pixel is enabled

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_mixed_pixel_enabled(self._ptr, get_error())


    def setMixedPixelEnabled(self, enabled: bool) -> None:
        """
        Set wether mixed pixel is enabled

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_mixed_pixel_enabled(self._ptr, enabled, get_error())


    def getMixedPixelLutFile(self) -> str:
        """
        Get location of the mixed pixel look up table file

        Returns
        ----------
        str
            File location
        """
        return lib.tof_processing_config_get_mixed_pixel_lut_file(self._ptr, get_error()).decode("utf-8")


    def setMixedPixelLutFile(self, file_location: str) -> None:
        """
        Set location of the mixed pixel look up table file

        Parameters
        ----------
        file_location : str
            File location
        """
        return lib.tof_processing_config_set_mixed_pixel_lut_file(self._ptr, file_location.encode("utf-8"), get_error())


    def getMixedPixelAmpMin(self) -> float:
        """
        Get minimum amplitude that pixel needs to have to apply mixed pixel to it

        Returns
        ----------
        float
            Minimum amplitude
        """
        return lib.tof_processing_config_get_mixed_pixel_amp_min(self._ptr, get_error())


    def setMixedPixelAmpMin(self, amp_min: float) -> None:
        """
        Set minimum amplitude that pixel needs to have to apply mixed pixel to it

        Parameters
        ----------
        amp_min : float
            Minimum amplitude
        """
        return lib.tof_processing_config_set_mixed_pixel_amp_min(self._ptr, amp_min, get_error())


    def getMixedPixelSortMode(self) -> MpSortMode:
        """
        Get what output is selected

        Returns
        ----------
        MpSortMode
            Sort mode
        """
        return lib.tof_processing_config_get_mixed_pixel_sort_mode(self._ptr, get_error())


    def setMixedPixelSortMode(self, sort_mode: MpSortMode) -> None:
        """
        Set what output is selected

        Parameters
        ----------
        sort_mode : MpSortMode
            Sort mode
        """
        return lib.tof_processing_config_set_mixed_pixel_sort_mode(self._ptr, sort_mode, get_error())


    def getMixedPixelAratio(self) -> float:
        """
        Get the amplitude ratio for detecting MPI

        Returns
        ----------
        float
            Amplitude ratio
        """
        return lib.tof_processing_config_get_mixed_pixel_aratio(self._ptr, get_error())


    def setMixedPixelAratio(self, aratio: float) -> None:
        """
        Set the amplitude ratio for detecting MPI

        Parameters
        ----------
        aratio : float
            Amplitude ratio
        """
        return lib.tof_processing_config_set_mixed_pixel_aratio(self._ptr, aratio, get_error())


    def getMixedPixelPhiDiff(self) -> float:
        """
        Get the phase difference for detecting MPI

        Returns
        ----------
        float
            Phase difference
        """
        return lib.tof_processing_config_get_mixed_pixel_phi_diff(self._ptr, get_error())


    def setMixedPixelPhiDiff(self, phi_diff: float) -> None:
        """
        Set the phase difference for detecting MPI

        Parameters
        ----------
        phi_diff : float
            Phase difference
        """
        return lib.tof_processing_config_set_mixed_pixel_phi_diff(self._ptr, phi_diff, get_error())


    def getMixedPixelAggressorThreshold(self) -> float:
        """
        Get the threshold to classify a pixel as an aggressor

        Returns
        ----------
        float
            Aggressor threshold
        """
        return lib.tof_processing_config_get_mixed_pixel_aggressor_threshold(self._ptr, get_error())


    def setMixedPixelAggressorThreshold(self, threshold: float) -> None:
        """
        Set the threshold to classify a pixel as an aggressor

        Parameters
        ----------
        threshold : float
            Aggressor threshold
        """
        return lib.tof_processing_config_set_mixed_pixel_aggressor_threshold(self._ptr, threshold, get_error())


    def getMixedPixelMinAggressorSize(self) -> int:
        """
        Get the minimum amount of bright pixels before a bright spot is classified
        as an aggressor

        Returns
        ----------
        int
            Minimum aggressor size
        """
        return lib.tof_processing_config_get_mixed_pixel_min_aggressor_size(self._ptr, get_error())


    def setMixedPixelMinAggressorSize(self, size: int) -> None:
        """
        Set the minimum amount of bright pixels before a bright spot is classified
        as an aggressor

        Parameters
        ----------
        size : int
            Minimum aggressor size
        """
        return lib.tof_processing_config_set_mixed_pixel_min_aggressor_size(self._ptr, size, get_error())


    def getMixedPixelMaxError3f(self) -> float:
        """
        Get the maximum error when using three modulation frequencies

        Returns
        ----------
        float
            The error
        """
        return lib.tof_processing_config_get_mixed_pixel_max_error_3f(self._ptr, get_error())


    def setMixedPixelMaxError3f(self, max_error_3f: float) -> None:
        """
        Set the maximum error when using three modulation frequencies

        Parameters
        ----------
        max_error_3f : float
            The error
        """
        return lib.tof_processing_config_set_mixed_pixel_max_error_3f(self._ptr, max_error_3f, get_error())


    def getMixedPixelAdaptiveAmpMax(self) -> float:
        """
        Get the maximum amplitude for adaptive sorting

        Returns
        ----------
        float
            Maximum amplitude
        """
        return lib.tof_processing_config_get_mixed_pixel_adaptive_amp_max(self._ptr, get_error())


    def setMixedPixelAdaptiveAmpMax(self, adaptive_amp_max: float) -> None:
        """
        Set the maximum amplitude for adaptive sorting

        Parameters
        ----------
        adaptive_amp_max : float
            Maximum amplitude
        """
        return lib.tof_processing_config_set_mixed_pixel_adaptive_amp_max(self._ptr, adaptive_amp_max, get_error())


    def getMixedPixelAdaptiveAmpMin(self) -> float:
        """
        Get the minimum amplitude for adaptive sorting

        Returns
        ----------
        float
            Minimum amplitude
        """
        return lib.tof_processing_config_get_mixed_pixel_adaptive_amp_min(self._ptr, get_error())


    def setMixedPixelAdaptiveAmpMin(self, adaptive_amp_min: float) -> None:
        """
        Set the minimum amplitude for adaptive sorting

        Parameters
        ----------
        adaptive_amp_min : float
            Minimum amplitude
        """
        return lib.tof_processing_config_set_mixed_pixel_adaptive_amp_min(self._ptr, adaptive_amp_min, get_error())


    def getMixedPixelAratioMin(self) -> float:
        """
        Get the minimum return amplitude ratio for adaptive sorting

        Returns
        ----------
        float
            Minimum return ratio
        """
        return lib.tof_processing_config_get_mixed_pixel_aratio_min(self._ptr, get_error())


    def setMixedPixelAratioMin(self, aratio_min: float) -> None:
        """
        Set the minimum return amplitude ratio for adaptive sorting

        Parameters
        ----------
        aratio_min : float
            Minimum return ratio
        """
        return lib.tof_processing_config_set_mixed_pixel_aratio_min(self._ptr, aratio_min, get_error())


    def getMixedPixelAggressorPhiOffset(self) -> float:
        """
        Get The maximum phase difference before a return is not classified as
        aggressor

        Returns
        ----------
        float
            Phase offset before not a aggressor return
        """
        return lib.tof_processing_config_get_mixed_pixel_aggressor_phi_offset(self._ptr, get_error())


    def setMixedPixelAggressorPhiOffset(self, aggressor_phi_offset: float) -> None:
        """
        Set The maximum phase difference before a return is not classified as
        aggressor

        Parameters
        ----------
        aggressor_phi_offset : float
            Phase offset before not a aggressor return
        """
        return lib.tof_processing_config_set_mixed_pixel_aggressor_phi_offset(self._ptr, aggressor_phi_offset, get_error())


    def getHdrEnabled(self) -> bool:
        """
        Get whether hdr is enabled. High dynamic range allows you to separate depth
        with high intensity difference. This is mostly useful when the sensor is
        being saturated.

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_hdr_enabled(self._ptr, get_error())


    def setHdrEnabled(self, enabled: bool) -> None:
        """
        Set whether hdr is enabled. High dynamic range allows you to separate depth
        with high intensity difference. This is mostly useful when the sensor is
        being saturated.

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_hdr_enabled(self._ptr, enabled, get_error())


    def getAverageEnabled(self) -> bool:
        """
        Get whether averaging is enabled. The average process allows you to average
        multiple frames together at the cost of frame rate

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_average_enabled(self._ptr, get_error())


    def setAverageEnabled(self, enabled: bool) -> None:
        """
        Set whether averaging is enabled. The average process allows you to average
        multiple frames together at the cost of frame rate

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_average_enabled(self._ptr, enabled, get_error())


    def getAverageNframes(self) -> int:
        """
        Get the number of frames to average together

        Returns
        ----------
        int
            Number of frames
        """
        return lib.tof_processing_config_get_average_nframes(self._ptr, get_error())


    def setAverageNframes(self, nframes: int) -> None:
        """
        Set the number of frames to average together

        Parameters
        ----------
        nframes : int
            Number of frames
        """
        return lib.tof_processing_config_set_average_nframes(self._ptr, nframes, get_error())


    def getTemporalEnabled(self) -> bool:
        """
        Get whether the temporal filter is enabled. The temporal filter is a running
        average as long as the values are within sigma standard deviations of each
        other

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_temporal_enabled(self._ptr, get_error())


    def setTemporalEnabled(self, enabled: bool) -> None:
        """
        Set whether the temporal filter is enabled. The temporal filter is a running
        average as long as the values are within sigma standard deviations of each
        other

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_temporal_enabled(self._ptr, enabled, get_error())


    def getTemporalSigma(self) -> float:
        """
        Get the number of standard deviations before resetting the running average

        Returns
        ----------
        float
            The standard deviation
        """
        return lib.tof_processing_config_get_temporal_sigma(self._ptr, get_error())


    def setTemporalSigma(self, sigma: float) -> None:
        """
        Set the number of standard deviations before resetting the running average

        Parameters
        ----------
        sigma : float
            The standard deviation
        """
        return lib.tof_processing_config_set_temporal_sigma(self._ptr, sigma, get_error())


    def getTemporalNframes(self) -> int:
        """
        Get the maximum number of frames to average together

        Returns
        ----------
        int
            Number of frames
        """
        return lib.tof_processing_config_get_temporal_nframes(self._ptr, get_error())


    def setTemporalNframes(self, nframes: int) -> None:
        """
        Set the maximum number of frames to average together

        Parameters
        ----------
        nframes : int
            Number of frames
        """
        return lib.tof_processing_config_set_temporal_nframes(self._ptr, nframes, get_error())


    def getGaussianEnabled(self) -> bool:
        """
        Get whether the gaussian blur is enabled. This is a standard gaussian blur
        applied over the depth image

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_gaussian_enabled(self._ptr, get_error())


    def setGaussianEnabled(self, enabled: bool) -> None:
        """
        Set whether the gaussian blur is enabled. This is a standard gaussian blur
        applied over the depth image

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_gaussian_enabled(self._ptr, enabled, get_error())


    def getGaussianSize(self) -> int:
        """
        Get the gaussian region size

        Returns
        ----------
        int
            Region size, 3x3 and 5x5 are supported
        """
        return lib.tof_processing_config_get_gaussian_size(self._ptr, get_error())


    def setGaussianSize(self, size: int) -> None:
        """
        Set the gaussian region size

        Parameters
        ----------
        size : int
            Region size, 3x3 and 5x5 are supported
        """
        return lib.tof_processing_config_set_gaussian_size(self._ptr, size, get_error())


    def getGaussianSigma(self) -> float:
        """
        Get the gaussian sigma

        Returns
        ----------
        float
            The standard deviation
        """
        return lib.tof_processing_config_get_gaussian_sigma(self._ptr, get_error())


    def setGaussianSigma(self, sigma: float) -> None:
        """
        Set the gaussian sigma

        Parameters
        ----------
        sigma : float
            The standard deviation
        """
        return lib.tof_processing_config_set_gaussian_sigma(self._ptr, sigma, get_error())


    def getMedianEnabled(self) -> bool:
        """
        Get whether the median filter is enabled. This is a standard median filter
        applied over the depth image

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_median_enabled(self._ptr, get_error())


    def setMedianEnabled(self, enabled: bool) -> None:
        """
        Set whether the median filter is enabled. This is a standard median filter
        applied over the depth image

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_median_enabled(self._ptr, enabled, get_error())


    def getMedianSize(self) -> int:
        """
        Get the median region size

        Returns
        ----------
        int
            Region size, 3x3 and 5x5 are supported
        """
        return lib.tof_processing_config_get_median_size(self._ptr, get_error())


    def setMedianSize(self, size: int) -> None:
        """
        Set the median region size

        Parameters
        ----------
        size : int
            Region size, 3x3 and 5x5 are supported
        """
        return lib.tof_processing_config_set_median_size(self._ptr, size, get_error())


    def getBilateralEnabled(self) -> bool:
        """
        Get whether the bilateral filter is enabled. This filter averages the
        current pixel with neighboring ones if they're within the specified standard
        deviation

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_bilateral_enabled(self._ptr, get_error())


    def setBilateralEnabled(self, enabled: bool) -> None:
        """
        Set whether the bilateral filter is enabled. This filter averages the
        current pixel with neighboring ones if they're within the specified standard
        deviation

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_bilateral_enabled(self._ptr, enabled, get_error())


    def getBilateralSize(self) -> int:
        """
        Get the bilateral region size

        Returns
        ----------
        int
            Region size, 3x3 and 5x5 are supported
        """
        return lib.tof_processing_config_get_bilateral_size(self._ptr, get_error())


    def setBilateralSize(self, size: int) -> None:
        """
        Set the bilateral region size

        Parameters
        ----------
        size : int
            Region size, 3x3 and 5x5 are supported
        """
        return lib.tof_processing_config_set_bilateral_size(self._ptr, size, get_error())


    def getBilateralSigma(self) -> float:
        """
        Get the bilateral standard deviation

        Returns
        ----------
        float
            The standard deviation
        """
        return lib.tof_processing_config_get_bilateral_sigma(self._ptr, get_error())


    def setBilateralSigma(self, sigma: float) -> None:
        """
        Set the bilateral standard deviation

        Parameters
        ----------
        sigma : float
            The standard deviation
        """
        return lib.tof_processing_config_set_bilateral_sigma(self._ptr, sigma, get_error())


    def getLocalMeansEnabled(self) -> bool:
        """
        Get Whether the local means filter is enabled.

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_local_means_enabled(self._ptr, get_error())


    def setLocalMeansEnabled(self, enabled: bool) -> None:
        """
        Set Whether the local means filter is enabled.

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_local_means_enabled(self._ptr, enabled, get_error())


    def getLocalMeansSize(self) -> int:
        """
        Get The local means region size

        Returns
        ----------
        int
            Region size
        """
        return lib.tof_processing_config_get_local_means_size(self._ptr, get_error())


    def setLocalMeansSize(self, size: int) -> None:
        """
        Set The local means region size

        Parameters
        ----------
        size : int
            Region size
        """
        return lib.tof_processing_config_set_local_means_size(self._ptr, size, get_error())


    def getLocalMeansSigma(self) -> float:
        """
        Get The local means standard deviation for how close values are

        Returns
        ----------
        float
            The stand deviation
        """
        return lib.tof_processing_config_get_local_means_sigma(self._ptr, get_error())


    def setLocalMeansSigma(self, sigma: float) -> None:
        """
        Set The local means standard deviation for how close values are

        Parameters
        ----------
        sigma : float
            The stand deviation
        """
        return lib.tof_processing_config_set_local_means_sigma(self._ptr, sigma, get_error())


    def getFlyingEnabled(self) -> bool:
        """
        Get whether flying pixel filter is enabled. The flying pixel filters out
        pixels when more than x edges are farther than x size away

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_flying_enabled(self._ptr, get_error())


    def setFlyingEnabled(self, enabled: bool) -> None:
        """
        Set whether flying pixel filter is enabled. The flying pixel filters out
        pixels when more than x edges are farther than x size away

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_flying_enabled(self._ptr, enabled, get_error())


    def getFlyingShape(self) -> FlyingShape:
        """
        Get the shape of which surrounding pixels are checked

        Returns
        ----------
        FlyingShape
            The shape
        """
        return lib.tof_processing_config_get_flying_shape(self._ptr, get_error())


    def setFlyingShape(self, shape: FlyingShape) -> None:
        """
        Set the shape of which surrounding pixels are checked

        Parameters
        ----------
        shape : FlyingShape
            The shape
        """
        return lib.tof_processing_config_set_flying_shape(self._ptr, shape, get_error())


    def getFlyingDistance(self) -> int:
        """
        Get the maximum distance allowed between surrounding pixels

        Returns
        ----------
        int
            Maximum distance
        """
        return lib.tof_processing_config_get_flying_distance(self._ptr, get_error())


    def setFlyingDistance(self, distance: int) -> None:
        """
        Set the maximum distance allowed between surrounding pixels

        Parameters
        ----------
        distance : int
            Maximum distance
        """
        return lib.tof_processing_config_set_flying_distance(self._ptr, distance, get_error())


    def getFlyingEdges(self) -> int:
        """
        Get the number of edges allowed to be larger than maximum distance

        Returns
        ----------
        int
            Maximum edges
        """
        return lib.tof_processing_config_get_flying_edges(self._ptr, get_error())


    def setFlyingEdges(self, edges: int) -> None:
        """
        Set the number of edges allowed to be larger than maximum distance

        Parameters
        ----------
        edges : int
            Maximum edges
        """
        return lib.tof_processing_config_set_flying_edges(self._ptr, edges, get_error())


    def getAmpThresholdEnabled(self) -> bool:
        """
        Get whether the amplitude threshold filter is enabled. This filter filters
        out pixels outside of the specified values

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_amp_threshold_enabled(self._ptr, get_error())


    def setAmpThresholdEnabled(self, enabled: bool) -> None:
        """
        Set whether the amplitude threshold filter is enabled. This filter filters
        out pixels outside of the specified values

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_amp_threshold_enabled(self._ptr, enabled, get_error())


    def getAmpThresholdMin(self) -> float:
        """
        Get the minimum amplitude allowed

        Returns
        ----------
        float
            Minimum value
        """
        return lib.tof_processing_config_get_amp_threshold_min(self._ptr, get_error())


    def setAmpThresholdMin(self, min: float) -> None:
        """
        Set the minimum amplitude allowed

        Parameters
        ----------
        min : float
            Minimum value
        """
        return lib.tof_processing_config_set_amp_threshold_min(self._ptr, min, get_error())


    def getAmpThresholdMax(self) -> float:
        """
        Get the maximum amplitude allowed

        Returns
        ----------
        float
            Maximum value
        """
        return lib.tof_processing_config_get_amp_threshold_max(self._ptr, get_error())


    def setAmpThresholdMax(self, max: float) -> None:
        """
        Set the maximum amplitude allowed

        Parameters
        ----------
        max : float
            Maximum value
        """
        return lib.tof_processing_config_set_amp_threshold_max(self._ptr, max, get_error())


    def getDistThresholdEnabled(self) -> bool:
        """
        Get whether the distance threshold filter is enabled. This filter filters
        out pixels outside of the specified values

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_dist_threshold_enabled(self._ptr, get_error())


    def setDistThresholdEnabled(self, enabled: bool) -> None:
        """
        Set whether the distance threshold filter is enabled. This filter filters
        out pixels outside of the specified values

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_dist_threshold_enabled(self._ptr, enabled, get_error())


    def getDistThresholdMin(self) -> float:
        """
        Get the minimum distance allowed

        Returns
        ----------
        float
            Minimum value
        """
        return lib.tof_processing_config_get_dist_threshold_min(self._ptr, get_error())


    def setDistThresholdMin(self, min: float) -> None:
        """
        Set the minimum distance allowed

        Parameters
        ----------
        min : float
            Minimum value
        """
        return lib.tof_processing_config_set_dist_threshold_min(self._ptr, min, get_error())


    def getDistThresholdMax(self) -> float:
        """
        Get the maximum distance allowed

        Returns
        ----------
        float
            Maximum value
        """
        return lib.tof_processing_config_get_dist_threshold_max(self._ptr, get_error())


    def setDistThresholdMax(self, max: float) -> None:
        """
        Set the maximum distance allowed

        Parameters
        ----------
        max : float
            Maximum value
        """
        return lib.tof_processing_config_set_dist_threshold_max(self._ptr, max, get_error())


    def getRadialScale(self) -> float:
        """
        Get how much to scale the radial value by

        Returns
        ----------
        float
            Radial scale
        """
        return lib.tof_processing_config_get_radial_scale(self._ptr, get_error())


    def setRadialScale(self, scale: float) -> None:
        """
        Set how much to scale the radial value by

        Parameters
        ----------
        scale : float
            Radial scale
        """
        return lib.tof_processing_config_set_radial_scale(self._ptr, scale, get_error())


    def getRadialAdd(self) -> float:
        """
        Get how much to add to the radial value

        Returns
        ----------
        float
            Radial addition
        """
        return lib.tof_processing_config_get_radial_add(self._ptr, get_error())


    def setRadialAdd(self, addition: float) -> None:
        """
        Set how much to add to the radial value

        Parameters
        ----------
        addition : float
            Radial addition
        """
        return lib.tof_processing_config_set_radial_add(self._ptr, addition, get_error())


    def getIntensityScale(self) -> float:
        """
        Get how much to scale the intensity value by

        Returns
        ----------
        float
            Intensity scale
        """
        return lib.tof_processing_config_get_intensity_scale(self._ptr, get_error())


    def setIntensityScale(self, scale: float) -> None:
        """
        Set how much to scale the intensity value by

        Parameters
        ----------
        scale : float
            Intensity scale
        """
        return lib.tof_processing_config_set_intensity_scale(self._ptr, scale, get_error())


    def getIntensityAdd(self) -> float:
        """
        Get how much to add to the intensity value

        Returns
        ----------
        float
            Intensity addition
        """
        return lib.tof_processing_config_get_intensity_add(self._ptr, get_error())


    def setIntensityAdd(self, addition: float) -> None:
        """
        Set how much to add to the intensity value

        Parameters
        ----------
        addition : float
            Intensity addition
        """
        return lib.tof_processing_config_set_intensity_add(self._ptr, addition, get_error())


    def getCharAdd(self) -> float:
        """
        Get the characterization addition component. The characterization function
        computes the sigma for the temporal and bilateral filter

        Returns
        ----------
        float
            addition
        """
        return lib.tof_processing_config_get_char_add(self._ptr, get_error())


    def setCharAdd(self, addition: float) -> None:
        """
        Set the characterization addition component. The characterization function
        computes the sigma for the temporal and bilateral filter

        Parameters
        ----------
        addition : float
            addition
        """
        return lib.tof_processing_config_set_char_add(self._ptr, addition, get_error())


    def getCharMult(self) -> float:
        """
        Get the characterization multiplication component. The characterization
        function computes the sigma for the temporal and bilateral filter

        Returns
        ----------
        float
            Multiplication
        """
        return lib.tof_processing_config_get_char_mult(self._ptr, get_error())


    def setCharMult(self, multiplication: float) -> None:
        """
        Set the characterization multiplication component. The characterization
        function computes the sigma for the temporal and bilateral filter

        Parameters
        ----------
        multiplication : float
            Multiplication
        """
        return lib.tof_processing_config_set_char_mult(self._ptr, multiplication, get_error())


    def getBinningEnabled(self) -> bool:
        """
        Get whether software binning is enabled

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_binning_enabled(self._ptr, get_error())


    def setBinningEnabled(self, enabled: bool) -> None:
        """
        Set whether software binning is enabled

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_binning_enabled(self._ptr, enabled, get_error())


    def getBinningSize(self) -> int:
        """
        Get the size of the area to bin, 2 would bin a 2x2 area

        Returns
        ----------
        int
            The bin size
        """
        return lib.tof_processing_config_get_binning_size(self._ptr, get_error())


    def setBinningSize(self, size: int) -> None:
        """
        Set the size of the area to bin, 2 would bin a 2x2 area

        Parameters
        ----------
        size : int
            The bin size
        """
        return lib.tof_processing_config_set_binning_size(self._ptr, size, get_error())


    def getBinningMode(self) -> BinningMode:
        """
        Get the binning mode to use

        Returns
        ----------
        BinningMode
            The binning mode
        """
        return lib.tof_processing_config_get_binning_mode(self._ptr, get_error())


    def setBinningMode(self, mode: BinningMode) -> None:
        """
        Set the binning mode to use

        Parameters
        ----------
        mode : BinningMode
            The binning mode
        """
        return lib.tof_processing_config_set_binning_mode(self._ptr, mode, get_error())


    def getBinningSigma(self) -> float:
        """
        Get The maximum standard deviation to use, only used in the smart binning
        mode

        Returns
        ----------
        float
            The standard deviation
        """
        return lib.tof_processing_config_get_binning_sigma(self._ptr, get_error())


    def setBinningSigma(self, sigma: float) -> None:
        """
        Set The maximum standard deviation to use, only used in the smart binning
        mode

        Parameters
        ----------
        sigma : float
            The standard deviation
        """
        return lib.tof_processing_config_set_binning_sigma(self._ptr, sigma, get_error())


    def getXyzDimension(self) -> XyzDimension:
        """
        Get the dimension of the xyz output

        Returns
        ----------
        XyzDimension
            The dimension
        """
        return lib.tof_processing_config_get_xyz_dimension(self._ptr, get_error())


    def setXyzDimension(self, dimension: XyzDimension) -> None:
        """
        Set the dimension of the xyz output

        Parameters
        ----------
        dimension : XyzDimension
            The dimension
        """
        return lib.tof_processing_config_set_xyz_dimension(self._ptr, dimension, get_error())


    def getRigidTransformationEnabled(self) -> bool:
        """
        Get whether rigid transformation is enabled

        Returns
        ----------
        bool
            Enabled
        """
        return lib.tof_processing_config_get_rigid_transformation_enabled(self._ptr, get_error())


    def setRigidTransformationEnabled(self, enabled: bool) -> None:
        """
        Set whether rigid transformation is enabled

        Parameters
        ----------
        enabled : bool
            Enabled
        """
        return lib.tof_processing_config_set_rigid_transformation_enabled(self._ptr, enabled, get_error())


    def getRigidTransformationMatrix(self) -> List[float]:
        """
        Get the rigid transformation 4x4 matrix

        Returns
        ----------
        List[float]
            4x4 transformation matrix
        """
        array = lib.tof_processing_config_get_rigid_transformation_matrix(self._ptr, get_error())
        return array.contents[:]


    def setRigidTransformationMatrix(self, matrix: List[float]) -> None:
        """
        Set the rigid transformation 4x4 matrix

        Parameters
        ----------
        matrix : List[float]
            4x4 transformation matrix
        """
        return lib.tof_processing_config_set_rigid_transformation_matrix(self._ptr, (ctypes.c_float * 16)(*matrix), get_error())


